@@ -1,3 +1,4 @@
+from django.conf import settings
 from django.utils import timezone
 from django.contrib.auth import get_user_model
 from django.db import models
@@ -12,6 +13,7 @@ from . import fields
 from django.forms import ValidationError
 from jionlp_time import parse_time
 from .utils.schedule_time import nlp_config_to_schedule_config
+from . import settings
 
 
 mdays = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
@@ -20,7 +22,7 @@ mdays = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
 UserModel = get_user_model()
 
 
-class Task(models.Model):
+class AbstractTask(models.Model):
     id = models.AutoField(primary_key=True)
     parent = models.ForeignKey('self', db_constraint=False, on_delete=models.DO_NOTHING,
                                null=True, blank=True, verbose_name='父任务')
@@ -44,6 +46,7 @@ class Task(models.Model):
         db_table = 'taskhub'
         verbose_name = verbose_name_plural = '任务中心'
         unique_together = ('name', 'user', 'parent')
+        abstract = True
 
     def __str__(self):
         return self.name
@@ -51,6 +54,13 @@ class Task(models.Model):
     __repr__ = __str__
 
 
+class Task(AbstractTask):
+
+    class Meta(AbstractTask.Meta):
+        swappable = 'TASK_MODEL'
+        abstract = 'django_common_task_system' not in settings.INSTALLED_APPS
+
+
 class TaskScheduleCallback(models.Model):
     id = models.AutoField(primary_key=True)
     name = models.CharField(max_length=100, verbose_name='回调')
@@ -69,6 +79,7 @@ class TaskScheduleCallback(models.Model):
         db_table = 'task_schedule_callback'
         verbose_name = verbose_name_plural = '任务回调'
         unique_together = ('name', 'user')
+        abstract = 'django_common_task_system' not in settings.INSTALLED_APPS
 
     def __str__(self):
         return self.name
@@ -79,6 +90,7 @@ class TaskScheduleCallback(models.Model):
 class ScheduleConfig:
 
     def __init__(self,
+                 base_on_now=True,
                  nlp_sentence=None,
                  schedule_type=None,
                  crontab=None,
@@ -93,6 +105,7 @@ class ScheduleConfig:
                  timing_datetime=None,
                  config=None,
                  **kwargs):
+        self.base_on_now = base_on_now
         self.nlp_sentence = nlp_sentence
         self.schedule_type = schedule_type
         self.once_schedule = once_schedule
@@ -112,6 +125,7 @@ class ScheduleConfig:
 
     def parse_config(self, config):
         schedule_type = self.schedule_type = config['schedule_type']
+        self.base_on_now = config.get('base_on_now', False)
         type_config = config[schedule_type]
         if schedule_type == TaskScheduleType.ONCE:
             self.once_schedule = type_config['schedule_start_time']
@@ -146,6 +160,7 @@ class ScheduleConfig:
             return config
         config = {
             'schedule_type': self.schedule_type,
+            'base_on_now': self.base_on_now,
         }
         schedule_type = self.schedule_type
         type_config: dict = config.setdefault(self.schedule_type, {})
@@ -192,7 +207,10 @@ class ScheduleConfig:
         return config
 
     def get_current_time(self, start_time=None):
-        now = max(timezone.now(), start_time) if start_time else timezone.now()
+        if self.base_on_now:
+            now = timezone.now()
+        else:
+            now = start_time or timezone.now()
         now_seconds = now.hour * 3600 + now.minute * 60 + now.second
         schedule_type = self.schedule_type
         type_config = self.config[schedule_type]
@@ -367,7 +385,7 @@ class ScheduleConfig:
 
 class TaskSchedule(models.Model):
     id = models.AutoField(primary_key=True)
-    task = models.ForeignKey(Task, on_delete=models.CASCADE, db_constraint=False, verbose_name='任务')
+    task = models.ForeignKey(settings.TASK_MODEL, on_delete=models.CASCADE, db_constraint=False, verbose_name='任务')
     priority = models.IntegerField(default=0, verbose_name='优先级')
     next_schedule_time = models.DateTimeField(default=timezone.now, verbose_name='下次运行时间', db_index=True)
     schedule_start_time = models.DateTimeField(default=datetime.min, verbose_name='开始时间')
@@ -398,6 +416,8 @@ class TaskSchedule(models.Model):
         db_table = 'task_schedule'
         verbose_name = verbose_name_plural = '任务计划'
         ordering = ('-priority', 'next_schedule_time')
+        unique_together = ('task', 'user')
+        abstract = 'django_common_task_system' not in settings.INSTALLED_APPS
 
     def __str__(self):
         return self.task.name
@@ -411,7 +431,7 @@ class TaskSchedule(models.Model):
         return self.priority > other.priority
 
 
-class TaskScheduleLog(models.Model):
+class AbstractTaskScheduleLog(models.Model):
     id = models.AutoField(primary_key=True)
     schedule = models.ForeignKey(TaskSchedule, db_constraint=False, on_delete=models.CASCADE, verbose_name='任务计划')
     status = common_fields.CharField(verbose_name='运行状态')
@@ -419,15 +439,19 @@ class TaskScheduleLog(models.Model):
     schedule_time = models.DateTimeField(verbose_name='计划时间')
     create_time = models.DateTimeField(default=timezone.now, verbose_name='创建时间')
 
-    @property
-    def finish_time(self):
-        return self.create_time
-
     class Meta:
         db_table = 'task_schedule_log'
         verbose_name = verbose_name_plural = '任务日志'
+        abstract = True
 
     def __str__(self):
         return "schedule: %s, status: %s" % (self.schedule, self.status)
 
     __repr__ = __str__
+
+
+class TaskScheduleLog(AbstractTaskScheduleLog):
+
+    class Meta(AbstractTaskScheduleLog.Meta):
+        swappable = 'TASK_SCHEDULE_LOG_MODEL'
+        abstract = 'django_common_task_system' not in settings.INSTALLED_APPS
